/*

Copyright (C) 2002-2003, Jeffrey D. Argast.

The authors make NO WARRANTY or representation, either express or implied,
with respect to this software, its quality, accuracy, merchantability, or
fitness for a particular purpose.  This software is provided "AS IS", and you,
its user, assume the entire risk as to its quality and accuracy.

Permission is hereby granted to use, copy, modify, and distribute this
software or portions thereof for any purpose, without fee, subject to these
conditions:

(1) If any part of the source code is distributed, then this
statement must be included, with this copyright and no-warranty notice
unaltered.

(2) Permission for use of this software is granted only if the user accepts
full responsibility for any undesirable consequences; the authors accept
NO LIABILITY for damages of any kind.

*/

#import "PerforceActionChangesSubmitted.h"
#import "PerforceSubmittedChange.h"
#import "AppDefaults.h"

@implementation PerforceActionChangesSubmitted

+ (void) defaultRunFor:(id)owner selector:(SEL)aSelector
{
	PerforceActionChangesSubmitted* action = [[[PerforceActionChangesSubmitted alloc] init] autorelease];
	
	[action runFor:owner selector:aSelector];
}

+ (void) defaultRunFor:(id)owner selector:(SEL)aSelector withPath:(NSString*)depotPath
{
	PerforceActionChangesSubmitted* action = [[[PerforceActionChangesSubmitted alloc] init] autorelease];
	
	[action runFor:owner selector:aSelector withPath:depotPath];
}

- (void) dealloc
{
	[fOwner release];
	[fChangesSubmittedArray release];
	
	[super dealloc];
}

- (void) processOutput: (NSString*) perforceOutput
{
	NSArray* split = [perforceOutput componentsSeparatedByString: @"\n"];
	NSMutableArray * result = [NSMutableArray arrayWithCapacity: [split count]];
	NSString * s;
	NSEnumerator * e = [split objectEnumerator];
	while (s = [e nextObject])
	{
		if ([s length] > 4)
			[result addObject: [PerforceSubmittedChange submittedChangeWithString: s]];
	}

	[fChangesSubmittedArray release];
	fChangesSubmittedArray = [[NSArray alloc] initWithArray: result];
}

- (void) runFor:(id)owner selector:(SEL)aSelector
{
	[self runFor:owner selector:aSelector withPath:nil];
}

- (void) runFor:(id)owner selector:(SEL)aSelector withPath:(NSString*)depotPath
{
    NSMutableArray* commandAndArgs = [[[NSMutableArray alloc] init] autorelease];
	
	fOwner    = [owner retain];
	fSelector = aSelector;

	[commandAndArgs addObject:@"changes"];
	
    [commandAndArgs addObject:@"-m"];
    
	AppDefaults* appDefaults = [AppDefaults defaultAppDefaults];

    [commandAndArgs addObject:[appDefaults getNumChangesToFetch]];
	
    [commandAndArgs addObject:@"-s"];
    
    [commandAndArgs addObject:@"submitted"];
	
	if ( (depotPath != nil) && ([depotPath length] > 0 ))
	{
		[commandAndArgs addObject:depotPath];
	}
    
    [self runAction:commandAndArgs stdInString:nil readToEOF:YES];
}

- (NSArray*) getChangesSubmitted
{
	return fChangesSubmittedArray;
}

@end
