/*

Copyright (C) 2002-2003, Jeffrey D. Argast.

The authors make NO WARRANTY or representation, either express or implied,
with respect to this software, its quality, accuracy, merchantability, or
fitness for a particular purpose.  This software is provided "AS IS", and you,
its user, assume the entire risk as to its quality and accuracy.

Permission is hereby granted to use, copy, modify, and distribute this
software or portions thereof for any purpose, without fee, subject to these
conditions:

(1) If any part of the source code is distributed, then this
statement must be included, with this copyright and no-warranty notice
unaltered.

(2) Permission for use of this software is granted only if the user accepts
full responsibility for any undesirable consequences; the authors accept
NO LIABILITY for damages of any kind.

*/

#import "PerforceActionEdit.h"
#import "AppUtils.h"
#import "MessageDefs.h"

static NSString* kNewLine = @"\n";

@implementation PerforceActionEdit

+ (void) defaultRun:(NSArray*) depotPaths
{
	PerforceActionEdit* action = [[[PerforceActionEdit alloc] init] autorelease];
	
	[action run:depotPaths changeNumber:@""];
}

+ (void) defaultRun:(NSArray*) depotPaths changeNumber:(NSString *)changeNumber
{
	PerforceActionEdit* action = [[[PerforceActionEdit alloc] init] autorelease];
	
	[action run:depotPaths changeNumber:changeNumber];
}

- (void) run: (NSArray*) depotPaths changeNumber: (NSString*) changeNumber
{
	fPathArray = [depotPaths retain];
	
    NSMutableArray* commandAndArgs = [[[NSMutableArray alloc] init] autorelease];
	
	NSMutableString* fileString = [[[NSMutableString alloc] init] autorelease];
	
	int n;
	int num = [depotPaths count];
	
	for ( n = 0; n < num; n++ )
	{
		[fileString appendString:[depotPaths objectAtIndex:n]];
		
		[fileString appendString:kNewLine];
	}
	
	[commandAndArgs addObject:@"-x"];
	
	[commandAndArgs addObject:@"-"];
	
	[commandAndArgs addObject:@"edit"];

	if (! [changeNumber isEqualTo:@""] )
	{
	    [commandAndArgs addObject:@"-c"];
	    [commandAndArgs addObject:changeNumber];
	}

    [self runAction:commandAndArgs stdInString:fileString readToEOF:NO];
}


- (void) dealloc
{
	[fOwner release];
	[fPathArray release];

	[super dealloc];
}


- (BOOL) isCriticalAction
{
	return YES;
}

- (void) notifyOwnerTaskIsComplete
{
	[super notifyOwnerTaskIsComplete];
	
	if ( [self wasSuccess] )
	{
		SendNotificationWithObject (kActionEditCommandComplete, fPathArray);
	}
}

@end
