/*

Copyright (C) 2002-2003, Jeffrey D. Argast.

The authors make NO WARRANTY or representation, either express or implied,
with respect to this software, its quality, accuracy, merchantability, or
fitness for a particular purpose.  This software is provided "AS IS", and you,
its user, assume the entire risk as to its quality and accuracy.

Permission is hereby granted to use, copy, modify, and distribute this
software or portions thereof for any purpose, without fee, subject to these
conditions:

(1) If any part of the source code is distributed, then this
statement must be included, with this copyright and no-warranty notice
unaltered.

(2) Permission for use of this software is granted only if the user accepts
full responsibility for any undesirable consequences; the authors accept
NO LIABILITY for damages of any kind.

*/

#import "PerforceActionFstat.h"
#import "PerforceFile.h"
#import "AppUtils.h"

#import "AppDefaults.h"

static NSString* kNewLine = @"\n";

@implementation PerforceActionFstat

+ (void) defaultRunFor:(id)owner
		 selector:(SEL)aSelector
		 withPath:(NSString*)depotPath
{
	PerforceActionFstat* action = [[[PerforceActionFstat alloc] init] autorelease];
	
	[action runFor:owner selector:aSelector withPath:depotPath];
}
		 
+ (void) defaultRunFor:(id)owner
		 selector:(SEL)aSelector
		 withPath:(NSString*)depotPath
		 includeDeletes:(BOOL)includeDeletes
{
	PerforceActionFstat* action = [[[PerforceActionFstat alloc] init] autorelease];
	
	[action runFor:owner selector:aSelector withPath:depotPath includeDeletes:includeDeletes];
}

+ (void) defaultRunFor:(id)owner
		 selector:(SEL)aSelector
		 withPaths:(NSArray*)depotPaths
		 privateData:(NSObject*)privateData
		 includeDeletes:(BOOL)includeDeletes
{
	PerforceActionFstat* action = [[[PerforceActionFstat alloc] init] autorelease];
	
	[action runFor:owner selector:aSelector withPaths:depotPaths privateData:privateData includeDeletes:includeDeletes];
}

		
- (void) dealloc
{
	[fOwner release];
	[fFilesArray release];
	[fPrivateData release];
	
	[super dealloc];
}

- (void) runFor:(id)owner selector:(SEL)aSelector withPath:(NSString*)dirPath
{
    BOOL showDeletedFiles = [[AppDefaults defaultAppDefaults] getShowDeletedFiles];
    
    [self runFor:owner selector:aSelector withPath:dirPath includeDeletes:showDeletedFiles];
}

- (void) runFor:(id)owner 
         selector:(SEL)aSelector 
		 withPath:(NSString*)dirPath
		 includeDeletes:(BOOL)includeDeletes
{
    BOOL showEntireDepot  = [[AppDefaults defaultAppDefaults] getShowEntireDepot];

    NSMutableArray* commandAndArgs = [[[NSMutableArray alloc] init] autorelease];
	
	fOwner			= [owner retain];
	fSelector		= aSelector;
	fIncludeDeletes	= includeDeletes;
	
	[commandAndArgs addObject:@"fstat"];
	
    if ( !showEntireDepot )
    {
		[commandAndArgs addObject:@"-C"];
    }
    
	[commandAndArgs addObject:dirPath];
    
	[self runAction:commandAndArgs stdInString:nil readToEOF:YES];
}

- (void) runFor:(id)owner 
         selector:(SEL)aSelector 
		 withPaths:(NSArray*)depotPaths
		 privateData:(NSObject*)privateData
		 includeDeletes:(BOOL)includeDeletes
{
    BOOL showEntireDepot  = [[AppDefaults defaultAppDefaults] getShowEntireDepot];

    NSMutableArray* commandAndArgs = [[[NSMutableArray alloc] init] autorelease];
	
	fOwner			= [owner retain];
	fSelector		= aSelector;
	fIncludeDeletes	= includeDeletes;
	fPrivateData	= [privateData retain];
	
	NSMutableString* fileString = [[[NSMutableString alloc] init] autorelease];
	
	int n;
	int num = [depotPaths count];
	
	if ( num == 0 )
	{
		NSLog(@"bad");
	}
	
	for ( n = 0; n < num; n++ )
	{
		[fileString appendString:[depotPaths objectAtIndex:n]];
		
		[fileString appendString:kNewLine];
	}
	
	[commandAndArgs addObject:@"-x"];
	
	[commandAndArgs addObject:@"-"];

	[commandAndArgs addObject:@"fstat"];
	
    if ( !showEntireDepot )
    {
		[commandAndArgs addObject:@"-C"];
    }
        
	[self runAction:commandAndArgs stdInString:fileString readToEOF:YES];
}


- (void) processOutput: (NSString*) perforceOutput
{
	// Each file is separated by two newlines.  
	NSArray* split = [[perforceOutput substringToIndex: [perforceOutput length] - 1] componentsSeparatedByString: @"\n\n"];

	NSMutableArray* newArray = [NSMutableArray arrayWithCapacity: [split count]];
	
	NSString* s;
	NSEnumerator* e = [split objectEnumerator];
	
	while (s = [e nextObject])
	{
		PerforceFile* perforceFile = [PerforceFile fileWithString: s];
		
		if ( [perforceFile isDeleted] )
		{
			if ( fIncludeDeletes )
			{
				[newArray addObject: perforceFile];
			}
		}
		else /*if ( ![perforceFile isAdd] )*/
		{
			[newArray addObject: perforceFile];
		}
	}
	
	[fFilesArray release];
	fFilesArray = [newArray retain];
}

static NSString* kNoSuchFile = @"no such file";
static NSString* kNotInClient = @"not in client";

- (void) processError: (NSString*) perforceOutput
{
	NSRange nsRange1 = [perforceOutput rangeOfString:kNoSuchFile];
	NSRange nsRange2 = [perforceOutput rangeOfString:kNotInClient];

	if ( (nsRange1.location == NSNotFound) && (nsRange2.location == NSNotFound) )
	{
		SendErrorMessage ([perforceOutput cString]);
	}
}

- (NSArray*) getFiles
{
	return fFilesArray;
}

- (NSObject*) getPrivateData
{
	return fPrivateData;
}

@end
