/*

Copyright (C) 2002-2003, Jeffrey D. Argast.

The authors make NO WARRANTY or representation, either express or implied,
with respect to this software, its quality, accuracy, merchantability, or
fitness for a particular purpose.  This software is provided "AS IS", and you,
its user, assume the entire risk as to its quality and accuracy.

Permission is hereby granted to use, copy, modify, and distribute this
software or portions thereof for any purpose, without fee, subject to these
conditions:

(1) If any part of the source code is distributed, then this
statement must be included, with this copyright and no-warranty notice
unaltered.

(2) Permission for use of this software is granted only if the user accepts
full responsibility for any undesirable consequences; the authors accept
NO LIABILITY for damages of any kind.

*/

#import "PerforceActionLabel.h"
#import "MessageDefs.h"
#import "AppUtils.h"


@implementation PerforceActionLabelNew

+ (void) defaultRunFor:(id)owner selector:(SEL)aSelector
{
	PerforceActionLabelNew* action = [[[PerforceActionLabelNew alloc] init] autorelease];
	
	[action runFor:owner selector:aSelector];
}

- (void) dealloc
{
	[fOwner release];

	[super dealloc];
}

- (void) runFor:(id)owner selector:(SEL)aSelector
{
    NSMutableArray* commandAndArgs = [[[NSMutableArray alloc] init] autorelease];
	
	fOwner    = [owner retain];
	fSelector = aSelector;

	[commandAndArgs addObject:@"label"];

	[commandAndArgs addObject:@"-o"];

    [self runAction:commandAndArgs stdInString:nil readToEOF:YES];
}

- (void) processOutput: (NSString*) perforceOutput
{
	// do nothing
}

@end

//
//
//

@implementation PerforceActionLabelGet

+ (void) defaultRunFor:(id)owner selector:(SEL)aSelector identifier:(NSString*)identifier
{
	PerforceActionLabelGet* action = [[[PerforceActionLabelGet alloc] init] autorelease];

	[action runFor:owner selector:aSelector identifier:identifier];
}

- (void) dealloc
{
	[fOwner release];
	[fIdentifier release];

	[super dealloc];
}

- (void) runFor:(id)owner selector:(SEL)aSelector identifier:(NSString*)identifier
{
    NSMutableArray* commandAndArgs = [[[NSMutableArray alloc] init] autorelease];
	
	fOwner    = [owner retain];
	fSelector = aSelector;
	fIdentifier = [identifier retain];

	[commandAndArgs addObject:@"label"];

	[commandAndArgs addObject:@"-o"];

	[commandAndArgs addObject:identifier];

    [self runAction:commandAndArgs stdInString:nil readToEOF:YES];
}

- (void) processOutput: (NSString*) perforceOutput
{
	// do nothing
}

- (NSString*) getIdentifier
{
	return fIdentifier;
}


@end

//
//
//

@implementation PerforceActionLabelDelete

+ (void) defaultRunFor:(id)owner selector:(SEL)aSelector identifier:(NSString*)identifier
{
	PerforceActionLabelDelete* action = [[[PerforceActionLabelDelete alloc] init] autorelease];

	[action runFor:owner selector:aSelector identifier:identifier];
}

- (void) dealloc
{
	[fOwner release];

	[super dealloc];
}

- (void) runFor:(id)owner selector:(SEL)aSelector identifier:(NSString*)identifier
{
    NSMutableArray* commandAndArgs = [[[NSMutableArray alloc] init] autorelease];
	
	fOwner    = [owner retain];
	fSelector = aSelector;

	[commandAndArgs addObject:@"label"];

	[commandAndArgs addObject:@"-d"];

	[commandAndArgs addObject:identifier];

    [self runAction:commandAndArgs stdInString:nil readToEOF:YES];
}


- (void) notifyOwnerTaskIsComplete
{
	[super notifyOwnerTaskIsComplete];
	
	if ( [self wasSuccess] )
	{
		SendNotification (kLabelChanged);
	}
}

@end

//
//
//

@implementation PerforceActionLabelIn

+ (void) defaultRunFor:(id)owner selector:(SEL)aSelector form:(NSString*)form
{
	PerforceActionLabelIn* action = [[[PerforceActionLabelIn alloc] init] autorelease];

	[action runFor:owner selector:aSelector form:form];
}

- (void) dealloc
{
	[fOwner release];
	[fForm release];

	[super dealloc];
}

- (void) runFor:(id)owner selector:(SEL)aSelector form:(NSString*)form
{
    NSMutableArray* commandAndArgs = [[[NSMutableArray alloc] init] autorelease];
	
	fOwner    = [owner retain];
	fSelector = aSelector;
	fForm	  = [form retain];
	
	[commandAndArgs addObject:@"label"];

	[commandAndArgs addObject:@"-i"];

    [self runAction:commandAndArgs stdInString:form readToEOF:YES sendStdInToConsole:NO];
}

- (NSString*) getForm
{
	return fForm;
}

- (void) notifyOwnerTaskIsComplete
{
	[super notifyOwnerTaskIsComplete];
	
	if ( [self wasSuccess] )
	{
		SendNotification (kLabelChanged);
	}
}

@end
